<?php
/**
 * SProf - Stupid Section Profiler
 */
class section_profile
{
	private $sections = array();
	private $current_name = false;
	/** total amount of time for all sections */
	public $total = 0;

	/**
	 * constructor
	 * @param $name the section name
	 */
	public function section_profile($name = false)
	{
		$this->init($name);
	}

	/**
	 * (Re-)initialization function
	 * @param $name section name
	 */
	public function init($name = false)
	{
		$this->sections = array();
		$this->current_name = false;

		if ($name !== false)
			$this->start($name);
	}

	/**
	 * Starts a timer with the given name.
	 * If $name is false, records duration since last
	 * start-time and re-starts the timer.
	 */
	public function start($name = false)
	{
		if ($name === $this->current_name)
			$this->stop($name);

		$this->current_name = $name;
		# it's a new section
		if (!isset($this->sections[$name]))
			$this->sections[$name] = array();

		# re-started with same section name
		if (!isset($this->sections[$name]['start']))
			$this->sections[$name]['start'] = gettimeofday(true);
	}

	/**
	 * Stops a timer and returns the duration.
	 * If $name is non-false, the section named
	 * $name is stopped instead.
	 */
	public function stop($name = false)
	{
		if ($name === false)
			$name = $this->current_name;

		if (!isset($this->sections[$name]['start']))
			return false;

		$duration = gettimeofday(true) - $this->sections[$name]['start'];
		$this->total += $duration;
		unset($this->sections[$name]['start']);
		if (!empty($this->sections[$name]['duration']))
			$duration += $this->sections[$name]['duration'];
		$this->sections[$name]['duration'] = $duration;
		$this->current_name = false;

		return $duration;
	}

	/**
	 * Finishes all sections and (optionally) prints them all
	 * @param $show if true, prints all sections runtime
	 */
	public function done($show = false)
	{
		if ($show)
			return $this->show();

		foreach ($this->sections as $name => $section) {
			if (isset($section['start']))
				$this->stop($name);
			if (!isset($section['duration'])) {
				die("A horrible error occurred\n");
				continue;
			}
		}

		return $this->total;
	}

	/**
	 * Lists all sections and their durations
	 */
	public function show()
	{
		foreach ($this->sections as $name => $section) {
			if (isset($section['start']))
				$this->stop($name);

			echo "$name: $section[duration]\n";
		}
		return $this->total;
	}
}
