<?php
require_once('devtools/tapunit.php');
require_once('api/api.inc.php');
require_once('classes/object_class.php');
if (empty($main_tap))
	$main_tap = new phptap();

class apiTest extends TapUnit
{
	private $api_command = 'OP5LIBCFG=test/phptap/configs/yml /usr/bin/php api/monitor.php';
	private function ensure_no_php_errors($output) {
		foreach ($output as $row) {
			$this->ok(strpos($row, 'Fatal') === false && strpos($row, 'thrown') === false && strpos($row, 'Stack') === false, 'Nothing that looks like a PHP error in output');
		}
	}
	/**
	* 	host objects, check that we are able to list hosts
	* 	Testing list_objects
	*/
	public function test_show_host_objects()
	{
		set_user('host_7', array('op5_admins'));
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->ok(is_array($return), 'output');
		$this->ok_eq('0', $status, 'exit code 0');
	}

	/**
	* 	host objects, check that we are able to list hosts when supplying a username
	* 	Testing list_objects
	*/
	public function test_show_host_objects_with_username()
	{
		exec($this->api_command.' -t host -a list_objects -u test_user', $return, $status);
		$this->ok(is_array($return), 'output');
		$this->ok_eq('0', $status, 'exit code 0');
	}

	/**
	* 	Add new host object
	*/
	public function test_add_host()
	{
		exec($this->api_command.' -t host -o host_name=API_TESTHOST -o alias=API_TESTHOST -o address=192.168.1.26 -o template=default-host-template -o contacts=contact1', $return, $status);
		$this->ok_empty($return, 'no output');
		$this->ok_eq('0', $status, 'exit code 0');
	}

	/**
	*	Check that our new host got added
	*/
	public function test_host_exists()
	{
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->ok(in_array('API_TESTHOST', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Rename host API_TESTHOST to API_TESTHOST_NEW
	*/
	public function test_rename_host()
	{
		exec($this->api_command.' -t host -n API_TESTHOST -R host_name=API_TESTHOST_NEW -o contacts=contact1,contact2', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Check that our renamed host exists
	*/
	public function test_renamed_host_exists()
	{
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->ok(in_array('API_TESTHOST_NEW', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Check if default-service template exists
	*/
	public function test_template_exists()
	{
		exec($this->api_command.' -t service_template -a list_objects', $return, $status);
		$this->ok(in_array('default-service', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Add service to API_TESTHOST_NEW
	*/
	public function test_add_service()
	{
		exec($this->api_command.' -t service -o template=default-service -o host_name=API_TESTHOST_NEW -o service_description=API_TEST_SERVICE -o check_command=check-host-alive', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Verify that service API_TEST_SERVICE was
	* 	added to host API_TESTHOST_NEW
	*/
	public function test_service_exists()
	{
		exec($this->api_command.' -t service -a list_objects', $return, $status);
		$this->ok(in_array('API_TESTHOST_NEW;API_TEST_SERVICE', $return), '');
		$this->ok_eq('0', $status, '');
	}

	public function test_service_on_hg_located()
	{
		exec($this->api_command.' -t service -a show_object -n "hostgroup_1;groupservice"', $return, $status);
		$this->ok_eq('0', $status, 'Showing hostgroup service should work');
		$this->ok(strpos(implode("\n", $return), "hostgroup_name=hostgroup_1") !== false, 'We should see hostgroup_name in output, got: '. implode("\n", $return));
	}

	/**
	 *	Add info to notes field in hostextinfo on host API_TESTHOST_NEW
	 */
	public function test_add_hostextinfo_note()
	{
		exec($this->api_command.' -t hostextinfo -o host_name=API_TESTHOST_NEW -o notes=API_TESTNOTE', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	 *  Remove hostextinfo notes field on host API_TESTHOST_NEW
	 *
	 *  (Objects without names such as hostextinfo aren't resolved, so this
	 *  creates SQL errors)
	 */
	public function test_remove_hostextinfo_note()
	{
		exec($this->api_command.' -t hostextinfo -o host_name=API_TESTHOST_NEW -o notes=', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	*	Remove API_TESTHOST_NEW and all its sub objects (1 service in this case)
	*/
	public function test_remove_host()
	{
		exec($this->api_command.' -t host -n API_TESTHOST_NEW -a delete', $return);
		# status can't be checked directly from  exec when doing delete. This is usually
		# because we might be in DEBUG mode which will result in a print
		# statement from the object_class. Checking with echo $? will do the trick.
		exec('echo $?', $status);
		$this->ok_eq('0', $status[0], '');
	}

	/**
	* 	Check that service API_TEST_SERVICE is removed
	*/
	public function test_service_notexists()
	{
		exec($this->api_command.' -t service -a list_objects', $return, $status);
		$this->ok_eq(false, in_array('API_TEST_SERVICE', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Check that host API_TESTHOST_NEW is removed
	*/
	public function test_host_notexists()
	{
		exec($this->api_command.' -t host -a list_objects', $return, $status);
		$this->ok_eq(false, in_array('API_TESTHOST_NEW', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Add new contact
	*/
	public function test_add_contact()
	{
		exec($this->api_command.' -t contact -o contact_name=API_CONTACT -o alias=API_CONTACT -o email=noname@domain.com -o service_notification_period=none -o host_notification_period=none', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	*	Verify that contact API_CONTACT exists
	*/
	public function test_contact_exists()
	{
		exec($this->api_command.' -t contact -a list_objects', $return, $status);
		$this->ok(in_array('API_CONTACT', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Remove contact API_CONTACT
	*/
	public function test_remove_contact()
	{
		exec($this->api_command.' -t contact -n API_CONTACT -a delete', $return);
		exec('echo $?', $status);
		$this->ok_eq('0', $status[0], '');
	}

	/**
	*	Verify that contact API_CONTACT was removed
	*/
	public function test_contact_notexists()
	{
		exec($this->api_command.' -t contact -a list_objects', $return, $status);
		$this->ok_eq(false, in_array('API_CONTACT', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	* 	Test that users can be added with permissions
	*/
	public function test_add_user_with_permissions()
	{
	    exec($this->api_command.' -t user -o Username=api_limited_user -o Password=api_limited_user -o "Access Levels=authorized_for_system_commands,authorized_for_configuration_information"', $return, $status);
	    $this->ok_empty($return, 'User was created without errors');
	    $this->ok_eq('0', $status, 'User add was successful');
	}

	/**
	* 	Test that newly created user exists
	*/
	public function test_user_with_permissions_exists()
	{
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->ok(in_array('api_limited_user', $return), '');
	    if (!in_array('api_limited_user', $return))
		var_dump($return);
	    $this->ok_eq(0, $status, '');
	}

	/**
	* 	Test that users group exists
	*/
	public function test_user_group_exists()
	{
	    exec($this->api_command.' -t usergroup -a list_objects', $return, $status);
	    $this->ok(in_array('user_api_limited_user', $return), 'Implicitly created usergroup is in group listing');
	    $this->ok_eq(0, $status, '');
	}

	/**
	* 	Test that users can be removed
	*/
	public function test_remove_user_with_permissions()
	{
	    exec($this->api_command.' -t user -n api_limited_user -a delete', $return, $status);
	    $this->ok_empty($return, 'User was deleted without errors');
	    $this->ok_eq('0', $status, 'User delete was successful');
	}

	/**
	* 	Test that users was removed
	*/
	public function test_user_with_permissions_notexists()
	{
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->ok(!in_array('api_limited_user', $return), 'User with permissions was deleted.');
	    $this->ok_eq(0, $status, 'User was deleted with no errors.');
	}

	/**
	* 	Test that users can be added
	*/
	public function test_add_user_with_groups()
	{
	    exec($this->api_command.' -t user -o Username=limited_user_with_groups -o Password=api_limited_user -o "groups=contact_limited"', $return, $status);
	    $this->ok_empty($return, 'User was created without errors');
	    $this->ok_eq('0', $status, 'User add was successful');
	}

	public function test_user_with_groups_exists()
	{
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->ok(in_array('limited_user_with_groups', $return), '');
	    $this->ok_eq(0, $status, '');
	}

	/**
	* 	Test that users can be removed
	*/
	public function test_remove_user_with_groups()
	{
	    exec($this->api_command.' -t user -n limited_user_with_groups -a delete', $return, $status);
	    $this->ok_empty($return, 'User was deleted without errors');
	    $this->ok_eq('0', $status, 'User delete was successful');
	}

	/**
	* 	Test that removed user doesn't exist
	*/
	public function test_user_with_groups_notexists()
	{
	    exec($this->api_command.' -t user -a list_objects', $return, $status);
	    $this->ok(!in_array('limited_user_with_groups', $return), '');
	    $this->ok_eq(0, $status, '');
	}

	/**
	* 	Test that groups can be added
	*/
	public function test_add_group() 
	{
		exec($this->api_command.' -t usergroup -o name=test_group -o access_rights=host_view_all,host_edit_all,service_view_all,service_edit_all,system_commands,configuration_information,system_information', $return, $status);
		$this->ok_empty($return, 'Group was created without errors');
		$this->ok_eq(0, $status, 'Group creation was successful');
	}

	/**
	* 	Test that group exist with expected permissions
	*/
	public function test_added_group_exists()
	{
		exec($this->api_command.' -t usergroup -a show_object -n test_group', $return, $status);
		$this->ok(strpos(implode("\n", $return), 'service_view_all') !== false, 'Expected permission flag was set on group.');
	}

	/**
	* 	Test that groups can be removed
	*/
	public function test_remove_added_group()
	{
		exec($this->api_command.' -t usergroup -n test_group -a delete', $return, $status);
		$this->ok_empty($return, 'Group was deleted without errors');
		$this->ok_eq('0', $status, 'Group delete was successful');
	}

	/**
	* 	Test that removed group doesn't exist
	*/
	public function test_group_notexist()
	{
	    exec($this->api_command.' -t usergroup -a list_objects', $return, $status);
	    $this->ok(!in_array('test_group', $return), 'Group was indeed deleted');
	    $this->ok_eq(0, $status, 'List groups was successful.');
	}

	/**
	*	Add new timeperiod API_TIMEPERIOD
	*/
	public function test_add_timeperiod()
	{
		exec($this->api_command.' -t timeperiod -o timeperiod_name=API_TIMEPERIOD -o alias=API_TIMEPERIOD', $return, $status);
		$this->ok_empty($return, '');
		$this->ok_eq('0', $status, '');
	}

	/**
	*	Verify that timeperiod API_TIMEPERIOD exists
	*/
	public function test_timeperiod_exists()
	{
		exec($this->api_command.' -t timeperiod -a list_objects', $return, $status);
		$this->ok(in_array('API_TIMEPERIOD', $return), '');
		$this->ok_eq('0', $status, '');
	}

	/**
	*	Remove timeperiod API_TIMEPERIOD
	*/
	public function test_remove_timeperiod()
	{
		exec($this->api_command.' -t timeperiod -n API_TIMEPERIOD -a delete', $return, $status);
		$this->ok_eq(0, $status, '');
	}

	/**
	*	Verify that API_TIMEPERIOD was deleted
	*/
	public function test_timeperiod_notexists()
	{
		exec($this->api_command.' -t timeperiod -a list_objects', $return, $status);
		$this->ok_eq(false, in_array('API_TIMEPERIOD', $return), '');
		$this->ok_eq('0', $status, '');
		#$this->markTestSkipped('MARKING TEST AS SKIPPED');
	}

	public function test_add_servicedependency()
	{
		exec("/usr/bin/php api/monitor.php -t servicedependency -o 'dependent_service=host_10;CPU Load' -o 'service=host_17;PING'", $return, $status);
		$this->ok(empty($return), 'Expected no output, but got '.implode("\n", $return));
		$this->ok_eq('0', $status, "Expected exit code 0, but was $status");
	}

	/**
	 * @FIXME: This will break on oracle, because it assumes IDs are reset
	 */
	public function test_servicedependency_exists()
	{
		exec("/usr/bin/php api/monitor.php -t servicedependency -a list_objects", $return, $status);
		$this->ok_eq(0, $status, '');
		$found = false;
		foreach ($return as $line) {
			if (strpos($line, '"host_10;CPU Load" depends on "host_17;PING" #') !== false) {
				$found = true;
			}
		}
		$this->ok($found, "Couldn't find added servicedependency: wanted \"host_10;CPU Load\" depends on \"host_17;PING\" in ".implode("\n", $return));
	}

	public function test_remove_servicedependency()
	{
		exec("/usr/bin/php api/monitor.php -t servicedependency -a delete -i 1", $return, $status);
		$this->ok_eq(0, $status, '');
		$this->ok(!in_array('"linux-server1;PING" depends on "service=printer1;PING" #1', $return), '');
	}

	/**
	 * @FIXME: This will break on oracle, because it assumes IDs are reset
	 */
	public function test_servicedependency_notexists()
	{
		exec("/usr/bin/php api/monitor.php -t servicedependency -a list_objects", $return, $status);
		$this->ok_eq(0, $status, '');
		$this->ok(!in_array('"linux-server1;PING" depends on "service=printer1;PING" #1', $return), '');
	}

	public function test_invalid_rel()
	{
		exec("/usr/bin/php api/monitor.php -t host -o host_name=API_TEST_HOST -o contacts=API_INVALID_CONTACT", $return, $status);
		$this->ok_eq(1, $status, 'Contact not found should cause invalid exit code');
		$found = false;
		foreach ($return as $row) {
			if (strpos($row, 'doesn\'t exist') !== false) {
				$found = true;
				break;
			}
		}
		$this->ok($found, 'Error message looks sane');
		$this->ensure_no_php_errors($return);
	}

	public function test_show_b0rked_obj_type()
	{
		$commands = array(
			$this->api_command.' -a show_object -n API_TEST_HOST',
			$this->api_command.' -a show_object -t host',
			$this->api_command.' -a show_object -t hosta -n API_TEST_HOST',
			$this->api_command.' -a list_objects'
		);
		foreach ($commands as $command) {
			exec($command, $return, $status);
			$this->ok_eq(1, $status, "Command $command returns error code 1");
			$this->ensure_no_php_errors($return);
		}
	}

	/**
	 * 	Test that API doesn't crash when listing objects
	 * Bug #5452
	 */
	public function test_object_listings()
	{
		global $Supported_Object_Types;
		$objtypes = $Supported_Object_Types;
		unset($objtypes[array_search('FILE', $objtypes)]);
		$objtypes[] = 'user';
		foreach ($objtypes as $objtype) {
			exec($this->api_command.' -t '.$objtype.' -a list_objects', $return, $status);
			$this->ok_eq('0', $status, 'Exit with OK status code');
		}
		if ($status != 0)
			print implode("\n", $return);
	}

	/**
	 * Check that api isn't directory dependent
	 * (checks for bug #4409 and similar cases)
	 */
	public function test_from_different_directory()
	{
		$curdir = getcwd();
		chdir('/tmp');
		exec("OP5LIBCFG=$curdir/test/phptap/configs/yml /usr/bin/php $curdir/api/monitor.php  -t timeperiod -o timeperiod_name=API_TIMEPERIOD -o alias=API_TIMEPERIOD", $return, $status);

		$this->ok_eq('0', $status, 'No error when adding timeperiod');
		unset($return);

		exec("OP5LIBCFG=$curdir/test/phptap/configs/yml /usr/bin/php $curdir/proccess_autoscan.php", $return, $status);
		$this->ok_empty($return, 'No output when running autoscan');
		$this->ok_eq('0', $status, 'No error when running autoscan');
		chdir($curdir);
	}

	/**
	 * Show changes
	 */
	public function test_show_changes()
	{
		pdo_exec_query('TRUNCATE TABLE changelog');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->ok_eq('0', $status, 'No error code from show changes');
		$this->ok_eq('No changes', $return[0], "'No changes' printed when there aren't any");
		$return = false;
		exec($this->api_command.' -t timeperiod -o timeperiod_name=API_TIMEPERIOD2 -o alias=API_TIMEPERIOD2', $return, $status);
		$this->ok_eq('0', $status, 'No error code when adding timeperiod');
		$this->ok_empty($return, 'Nothing printed when adding timeperiod');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->ok_eq('0', $status, 'No error code from show changes');
		$this->ok(!empty($return), 'Something printed when showing changed timeperiod');
	}

	/**
	 *	Undo config
	 */
	public function test_undo_config()
	{
		exec($this->api_command.' -a undo_config', $return, $status);
		$this->ok_eq('0', $status, 'No error code from undo config');
		$this->ok_empty($return, 'No output when undoing config');
		exec($this->api_command.' -a show_changes', $return, $status);
		$this->ok_eq('0', $status, 'No error code when examining empty changelog');
		$this->ok_eq('No changes', trim($return[0]), 'Message showing that the changelog is empty is printed');
	}

	/**
	 * We somewhat magically default the username of any non-authed user to
	 * "superuser" - this shouldn't end up in any changelogs if the user did
	 * provide a different username
	 */
	public function test_edit_object_with_username()
	{
		exec($this->api_command.' -t host -o host_name=API_TESTHOST -o template=default-host-template -u test_user', $return, $status);
		$this->ok(is_array($return), 'output');
		$this->ok_eq('0', $status, 'exit code 0');

		exec($this->api_command.' -a show_changes', $return, $status);
		$return = implode("\n", $return);
		$this->ok_eq('0', $status, 'No error code when examining changelog');
		$this->ok(strpos($return, 'API_TESTHOST') !== false, 'Newly created host is in changelog: '.$return);
		$this->ok(strpos($return, 'test_user') !== false, 'Username used for creating host is in changelog: '.$return);
		$this->ok(strpos($return, 'superuser') === false, 'Global magical super-username is not in chanelog: '.$return);

		exec($this->api_command.' -a undo_config', $return, $status);
		$this->ok_eq('0', $status, 'No error code from undo config');
		$this->ok_empty($return, 'No output when undoing config');
	}

}

runUnit('apiTest', $main_tap);
