<?php
require_once('devtools/tapunit.php');
require_once('classes/object_class.php');
if (empty($main_tap))
	$main_tap = new phptap();

class CacheTest extends TapUnit
{
	function setUp()
	{
		set_user('host_7', array('op5_admins'));
	}

	function testIfCacheWhenRelationChanges()
	{
		pdo_exec_query('TRUNCATE TABLE changelog');
		$changes = new Changelog();
		$this->ok_eq($changes->is_recache_required(), false, 'Recache is not required when no changes');
		$host = new host_class(32);
		$this->ok(is_numeric($host->id), 'Host exists');
		$contact = new contact_class(5);
		$this->ok(is_numeric($contact->id), 'Contact exists');
		$host->obj['contacts'][] = $contact->id;
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'User can save');
		$changes = new Changelog();
		$this->ok_eq($changes->is_recache_required(), true, 'Recache is required when host has new contact');
		
		revert_users_changes();
	}

	function testIfCacheWhenNewObject()
	{
		pdo_exec_query('TRUNCATE TABLE changelog');
		$cg = new contactgroup_class();
		$cg->obj['contactgroup_name'] = 'contactgroup_name';
		$cg->obj['alias'] = 'contactgroup_name';
		$cg->obj['members'] = array(1);
		$cg->save_object();
		$this->ok_empty($cg->validation_error_list, 'User can save');
		$changes = new Changelog();
		$this->ok_eq($changes->is_recache_required(), true, 'Recache is required when new contactgroup');
		revert_users_changes();
	}

	function testIfCacheWhenUnrelatedChanges()
	{
		pdo_exec_query('TRUNCATE TABLE changelog');
		$c = new command_class();
		$c->obj['command_name'] = 'command_name';
		$c->obj['command_line'] = 'a';
		$c->save_object();
		$this->ok_empty($c->validation_error_list, 'User can save');

		$hg = new hostgroup_class(4);
		$hg->obj['members'][] = 3;
		$hg->save_object();
		$this->ok_empty($hg->validation_error_list, 'User can save');

		$changes = new Changelog();
		$this->ok_eq($changes->is_recache_required(), false, 'Recache is not required when unrelated changes');
		revert_users_changes();
	}
}

runUnit('CacheTest', $main_tap);

?>
