<?php
require_once('devtools/tapunit.php');
require_once('classes/object_class.php');
if (empty($main_tap))
	$main_tap = new phptap();

/**
 * Tests to make sure an object's related objects and relations to other
 * objects are deleted with the object itself.
 * This is relevant to test, as errors here have been known to make nacoma
 * completely unable to export it's settings.
 */
class DeleteTest extends TapUnit
{
	static function setUpClass()
	{
		set_user('host_7', array('op5_admins'));
	}

	function setUp()
	{
		$this->host = new host_class();
		$this->host->get_default_object();
		$this->host->obj['host_name'] = 'dead object';
		$this->host->obj['alias'] = 'dead object';
		$this->host->obj['address'] = 'address';
		$this->service = new service_class();
		$this->service->get_default_object();
		$this->service->obj['service_description'] = 'dead';
		$this->service->obj['check_command'] = 1;
	}

	function relationTest($obj, $rels) {
		$obj_id = $obj->id;
		foreach ($rels as $rel => $cnt) {
			$res = pdo_exec_query('SELECT 1 FROM '.get_junction_table_name($obj->type, $rel)." WHERE $obj->type=$obj_id");
			$this->ok_eq(count($res->fetchAll(PDO::FETCH_NUM)), $cnt, "Expected number of $obj->type -> $rel relations");
			unset($res);
		}
	}


	function saveAndTest($obj, $rels=array()) {
		$this->ok($obj->save_object(), 'Object saved');
		$this->ok_empty($obj->validation_error_list, "Can save object");
		$obj_id = $obj->id;
		$res = pdo_exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->ok_eq(count($res->fetchAll(PDO::FETCH_NUM)), 1, "Found the $obj->type object");
		$this->relationTest($obj, $rels);
	}
	function deleteAndTest($obj, $rels=array()) {
		$obj_id = $obj->id;
		$res = $obj->banish_object();
		if (!$res)
			$this->fail("Couldn't delete object: ".$obj->get_deletion_message());
		else
			$this->pass("Deleted object");
		$res = pdo_exec_query("SELECT 1 FROM $obj->db_table WHERE id=$obj_id");
		$this->ok_eq(count($res->fetchAll(PDO::FETCH_NUM)), 0, "$obj->type object was deleted");
		foreach ($rels as $rel => $cnt)
			$rels[$rel] = 0;
		$this->relationTest($obj, $rels);
	}

	function testDeleteHostWithRelations()
	{
		$this->host->obj['parents'] = array(36);
		$this->host->obj['hostgroups'] = array(1);
		$this->host->obj['contacts'] = array(1, 2);
		$this->host->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
		$this->deleteAndTest($this->host, array('contacts' => 2, 'contact_groups' => 1, 'parents' => 1, 'hostgroup'=> 1));
	}

	function testDeleteServiceWithRelations()
	{
		$member = new host_class('host_9');
		$this->ok(is_numeric($member->id), 'Member host exists');
		$this->service->obj['contacts'] = array(1,2);
		$this->service->obj['contact_groups'] = array(1);
		$this->service->obj['servicegroups'] = array(1);
		$this->service->obj['host_name'] = $member->id;
		$this->saveAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
		$this->deleteAndTest($this->service, array('contacts' => 2, 'contact_groups' => 1, 'servicegroup' => 1));
	}

	function testDeleteHostGroupWithRelations()
	{
		$member = new host_class('host_9');
		$this->ok(is_numeric($member->id), 'Member host exists');
		$hg = new hostgroup_class();
		$hg->obj['hostgroup_name'] = 'name';
		$hg->obj['alias'] = 'name';
		$hg->obj['members'] = array($member->id);
		$this->ok($hg->save_object(), 'Hostgroup saved');
		$this->ok_empty($hg->validation_error_list, "No errors saving hostgroup");
		$this->host->obj['hostgroups'] = array($hg->id);
		$this->ok($this->host->save_object(), 'Host saved');
		$this->ok_empty($this->host->validation_error_list, "No errors saving host");
		$hg = new hostgroup_class($hg->id);
		$this->saveAndTest($hg, array('members' => 2));
		$this->deleteAndTest($hg, array('members' => 2));

		$this->host->banish_object();
	}

	function testDeleteHostWithService()
	{
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->service->obj['contacts'] = array(1, 2);
		$this->service->obj['contact_groups'] = array(1);
		$this->saveAndTest($this->service, array('contacts'=>2, 'contact_groups'=>1));
		$this->host->banish_object();
		$res = pdo_exec_query("SELECT 1 FROM service WHERE service_description = 'dead'");
		$this->ok_empty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		$this->relationTest($this->service, array('contact'=>0,'contactgroup'=>0));
	}

	function testDeleteHostWithServiceEscalation()
	{
		$this->saveAndTest($this->host);
		$this->service->obj['host_name'] = $this->host->id;
		$this->saveAndTest($this->service);
		$sesc = new serviceescalation_class();
		$sesc->get_default_object();
		$sesc->obj['service'] = $this->service->id;
		$sesc->obj['contacts'] = array(1,2);
		$this->saveAndTest($sesc, array('contacts'=>2));
		$this->host->banish_object();
		$res = pdo_exec_query("SELECT 1 FROM service WHERE id=".$this->service->id);
		$this->ok_empty($res->fetchAll(PDO::FETCH_NUM), 'Service is gone');
		unset($res);
		$res = pdo_exec_query("SELECT 1 FROM serviceescalation WHERE id=".$sesc->id);
		$this->ok_empty($res->fetchAll(PDO::FETCH_NUM), 'Serviceescalation is gone');
		$this->relationTest($sesc, array('contacts'=>0));
	}

	function testDeleteWithReqAltDeps()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->ok_empty($hg->validation_error_list, "New hostgroup saved ok");
		$this->service->set_master_object_type('hostgroup');
		$this->service->obj['hostgroup_name'] = $hg->id;
		$this->service->save_object();
		$this->ok_empty($this->service->validation_error_list, 'New service saved ok');
		$hg->banish_object();
		$res = pdo_exec_query('SELECT COUNT(1) FROM service WHERE id = '.$this->service->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->ok_eq($row[0], 0, "Service was deleted when it's parent object was");
	}

	function testDeleteHostDeps()
	{
		$hg = new hostgroup_class();
		$hg->get_default_object();
		$hg->obj['hostgroup_name'] = 'test hostgroup';
		$hg->obj['alias'] = 'test hostgroup';
		$hg->obj['members'] = array(43);
		$hg->save_object();
		$this->ok_empty($hg->validation_error_list, "New hostgroup saved ok");

		$host = new host_class();
		$host->get_default_object();
		$host->obj['host_name'] = 'test object';
		$host->obj['alias'] = 'test object';
		$host->obj['address'] = 'address';
		$host->save_object();
		$this->ok_empty($host->validation_error_list, 'New host saved ok');
		
		$this->host->save_object();
		$this->ok_empty($this->host->validation_error_list, 'New host saved ok');

		$hd = new hostdependency_class();
		$hd->get_default_object();
		$hd->obj['host_name'] = $host->id;
		$hd->obj['hostgroup_name'] = $hg->id;
		$hd->obj['dependent_host_name'] = $this->host->id;
		$hd->save_object();
		$this->ok_empty($hd->validation_error_list, 'New hostdependency saved ok');

		$hgobj = $hg->obj;
		$hg->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->ok(!empty($nhd->obj), 'Hostdependency still exists');
		$res = pdo_exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->ok_empty($row['hostgroup_name'], 'Hostgroup cleared in dependency when deleted');
		$this->ok_eq($row['host_name'], $host->id, 'Host not cleared in dependency');
		unset($res);

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->ok_empty($nhd->obj, "Hostdependency doesn't still exist");
		$res = pdo_exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->ok_eq($row[0], 0, "Hostgroup deleted when both it's req_alts are");
		unset($res);

		$hg->set_object_id('new');
		$hg->obj = $hgobj;
		$hg->save_object();
		$hd->obj['hostgroup_name'] = $hg->id;
		$host->set_object_id('new');
		$host->obj = $hostobj;
		$host->save_object();
		$hd->obj['host_name'] = $host->id;
		$hd->save_object();
		$this->ok_empty($hd->validation_error_list, 'Hostdependency updated ok');

		$hostobj = $host->obj;
		$host->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->ok(!empty($nhd->obj), 'Hostdependency still exists');
		$res = pdo_exec_query('SELECT host_name, hostgroup_name FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_ASSOC);
		$this->ok_empty($row['host_name'], 'Host cleared in dependency when deleted');
		$this->ok_eq($row['hostgroup_name'], $hg->id, 'Hostgroup not cleared in dependency');
		unset($res);

		$hg->banish_object();
		$nhd = new hostdependency_class($hd->id);
		$this->ok_empty($nhd->obj, "Hostdependency doesn't still exist");
		$res = pdo_exec_query('SELECT COUNT(1) FROM hostdependency WHERE id = '.$hd->id);
		$row = $res->fetch(PDO::FETCH_NUM);
		$this->ok_eq($row[0], 0, "Hostdependency deleted when both it's req_alts are");
		unset($res);
		$this->host->banish_object();
	}
}

runUnit('DeleteTest', $main_tap);

?>
