<?php

require_once('devtools/tapunit.php');
require_once('classes/object_class.php');

class SpecialConfigs extends TapUnit
{
	function setUp()
	{
		global $slave_obj_array, $Object_Index_Table;
		global $Object_Reference_List, $Reverse_Object_Reference_table;
		global $import_warnings, $import_errors, $index, $num_objs, $obj_array;
		global $master_list;
		global $custom_vars, $Template_Cache;
		$Object_Index_Table = false;
		$Objects_For_User = false;
		$Object_Reference_List = false;
		$Reverse_Object_Reference_Table = array();
		$master_list = false;
		$slave_obj_array = array();
		$import_warnings = array();
		$import_errors = array();
		$index = array();
		$num_objs = array();
		$obj_array = array();
		$custom_vars = array();
		$Template_Cache = array();
	}

	function config_equal($pre, $post)
	{
		$empty_file = array("###############################################################################\n",
		                    "#  Generated by op5 Monitor webconfiguration exporter\n",
		                    "#\n",
		                    "# Exported <date> by <user>\n",
		                    "#\n\n");
		if ($pre === null)
			$pre = $empty_file;
		else
			$pre = file($pre);
		if ($post === null)
			$post = $empty_file;
		else
			$post = file($post);
		# line 4 is the one with the export date. Unset:
		if (isset($pre[3]))
			unset($pre[3]);
		if (isset($post[3]))
			unset($post[3]);
		$pre = implode("", $pre);
		$post = implode("", $post);
		$this->ok_eq($pre, $post, 'Config file is the same after import and export');
	}

	/**
	 * When there was only one serviceextinfo, and that one serviceextinfo didn't
	 * have a parent, things used to break.
	 */
	function testImportBuggyServiceextinfo()
	{
		# disable asserts during this test - the hostextinfo triggers (quite correct) errors
		assert_options(ASSERT_ACTIVE, 0);
		$cfg = 'test/phptap/configs/minimal-buggy-extinfo.cfg';
		$this->ok(import_config('phptap', $cfg), "import_config('phptap', '$cfg')");
		assert_options(ASSERT_ACTIVE, 1);
	}

	function testExportAfterDelete()
	{
		$empty_pre = 'test/phptap/configs/deleted-dependencies.cfg';
		$hg_dep_pre = 'test/phptap/configs/related-dependencies-hg.cfg';
		$host_dep_pre = 'test/phptap/configs/related-dependencies-host.cfg';
		$pre_cfg = array($empty_pre, $hg_dep_pre, $host_dep_pre);
		$post_cfg = array();
		foreach ($pre_cfg as $cfg) {
			$post_cfg[] = $cfg.'.new';
		}
		$this->ok(import_config('phptap', $pre_cfg), "import_config('phptap', '$pre_cfg')");
		$host = new host_class('the_host');
		$this->ok($host->id && is_numeric($host->id), 'Host was found');
		$host->banish_object();
		$cg = new contactgroup_class('the_contactgroup');
		$this->ok($cg->id && is_numeric($cg->id), 'Contactgroup was found');
		$cg->banish_object();

		foreach ($post_cfg as $cfg)
			touch($cfg);

		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->config_equal($empty_pre.'.new', null);
		$this->config_equal($hg_dep_pre.'.new', $hg_dep_pre.'.expected');
		$this->config_equal($host_dep_pre.'.new', $host_dep_pre.'.expected');
	}

	/**
	 * Test for bug #4999
	 */
	function testTrailingComma()
	{
		$pre = 'test/phptap/configs/trailing-comma.cfg';
		$post = 'test/phptap/configs/trailing-comma.cfg.expected';
		$this->ok(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$ret = do_export(true);
		$this->ok_id($ret, true, 'Config export was successful');
		$this->config_equal($pre.'.new', $post);
	}

	/**
	 * Test for bug #5105
	 */
	function testMinimizeConfig()
	{
		$pre = 'test/phptap/configs/minimize-template.cfg';
		$post = 'test/phptap/configs/minimize-template.cfg.expected';
		$this->ok(import_config('phptap', $pre), "import_config('phptap', '$pre')");

		touch($pre.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$host = new host_class('test');
		$this->ok($host->save_object(), 'Save object is ok');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->config_equal($pre.'.new', $post);
	}
	
	/**
	 * The bug is that get_object can't fetch any more relations than it fetches objects
	 */
	function testChangeUnrelated()
	{
		$pre_base = 'test/phptap/configs/hostwithservices.cfg';
		$pre_deps = 'test/phptap/configs/separate_service.cfg';
		$this->ok(import_config('phptap', array($pre_base, $pre_deps)), "import_config('phptap', array($pre_base, $pre_deps))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_base.'.new');
		touch($pre_deps.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$svc = new service_class('base_host;separate_service');
		$svc->obj['notes'] = 'foo';
		$this->ok($svc->save_object(), 'Save object is ok');
		$this->ok_empty($svc->validation_error_list, 'No save failures');
		unset($svc->obj['notes']);
		$this->ok($svc->save_object(), 'Save object is ok');
		$this->ok_empty($svc->validation_error_list, 'No save failures');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_deps.'.new', $pre_deps);
	}

	function testChangeChild()
	{
		$pre_first = 'test/phptap/configs/emptyhostinafile1.cfg';
		$pre_second = 'test/phptap/configs/emptyhostinafile2.cfg';
		$this->ok(import_config('phptap', array($pre_first, $pre_second)), "import_config('phptap', array($pre_first, $pre_second))");
		global $IMPORTING;
		$IMPORTING=false;

		touch($pre_first.'.new');
		touch($pre_second.'.new');
		$query = "UPDATE file_tbl SET file_name=concat(file_name, '.new')";
		pdo_exec_query($query);

		$host1 = new host_class('hostinfile1');
		$this->ok(!empty($host1->obj), 'host has data');

		$host2 = new host_class('hostinfile2');
		$this->ok(!empty($host2->obj), 'host has data');

		$host2->obj['parents'] = array($host1->id);
		$this->ok($host2->save_object(), 'host with parents saved');
		$this->ok_empty($host2->validation_error_list, 'Error list was empty');
		$host2 = new host_class('hostinfile2');
		$this->ok_eq($host2->obj['parents'], array($host1->id), 'Parent is still there after saving');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$host1 = new host_class('hostinfile1');
		$this->ok_eq($host1->obj['children'], array($host2->id), 'Parent has child after exporting');

		$host1->obj['host_name'] = 'hostinfilewithnewname';
		$this->ok($host1->save_object(), 'host with parents saved');
		$this->ok_empty($host1->validation_error_list, 'Error list was empty');

		$changes = new Changelog();
		$ret = do_export($changes);
		$this->ok_id(true, $ret, "Export was successful");
		$this->config_equal($pre_first.'.new', $pre_first.'.expected');
		$this->config_equal($pre_second.'.new', $pre_second.'.expected');
	}
}

if (empty($main_tap))
	$main_tap = new phptap('Object tests');

runUnit('SpecialConfigs', $main_tap);
